#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Bike Pattern
# 
# We used a little bit of machine learning on [Divvy Data](https://www.divvybikes.com/system-data) to dig into a better division of Chicago. We try to identify patterns among bike stations.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


get_ipython().run_line_magic('matplotlib', 'inline')


# ## The data
# 
# [Divvy Data](https://www.divvybikes.com/system-data) publishes a sample of the data. 

# In[3]:


from pyensae.datasource import download_data
file = download_data("Divvy_Trips_2016_Q3Q4.zip", url="https://s3.amazonaws.com/divvy-data/tripdata/")


# We know the stations.

# In[4]:


import pandas
stations = pandas.read_csv("Divvy_Stations_2016_Q3.csv")
bikes = pandas.concat([pandas.read_csv("Divvy_Trips_2016_Q3.csv"),
                       pandas.read_csv("Divvy_Trips_2016_Q4.csv")])


# In[5]:


from datetime import datetime, time
df = bikes
df["dtstart"] = pandas.to_datetime(df.starttime, infer_datetime_format=True)
df["dtstop"] = pandas.to_datetime(df.stoptime, infer_datetime_format=True)
df["stopday"] = df.dtstop.apply(lambda r: datetime(r.year, r.month, r.day))
df["stoptime"] = df.dtstop.apply(lambda r: time(r.hour, r.minute, 0))
df["stoptime10"] = df.dtstop.apply(lambda r: time(r.hour, (r.minute // 10)*10, 0))  # every 10 minutes


# In[6]:


df['stopweekday'] = df['dtstop'].dt.dayofweek


# ## Normalize per week day (stop)

# In[7]:


df.columns


# In[8]:


key = ["to_station_id", "to_station_name", "stopweekday", "stoptime10"]
keep = key + ["trip_id"]
aggtime = df[keep].groupby(key, as_index=False).count()
aggtime.columns = key + ["nb_trips"]
aggtime.head()


# In[9]:


aggday = df[keep[:-2] + ["trip_id"]].groupby(key[:-1], as_index=False).count()
aggday.columns = key[:-1] + ["nb_trips"]
aggday.sort_values("nb_trips", ascending=False).head()


# In[10]:


merge = aggtime.merge(aggday, on=key[:-1], suffixes=("", "day"))
merge.head()


# In[11]:


merge["dist"] = merge["nb_trips"] / merge["nb_tripsday"]
merge.head()


# In[12]:


merge[merge["to_station_id"] == 2].plot(x="stoptime10", y="dist", figsize=(14,4), kind="area");


# ## Clustering (stop)
# 
# We cluster these distribution to find some patterns. But we need vectors of equal size which should be equal to 24*6.

# In[13]:


print(key)
merge.groupby(key[:-1], as_index=False).count().head()


# In[14]:


from ensae_projects.datainc.data_bikes import add_missing_time
full = df = add_missing_time(merge, delay=10, column="stoptime10", values=["nb_trips", "nb_tripsday", "dist"])
df.groupby(key[:-1], as_index=False).count().head()


# This is much better.

# In[15]:


df[df["to_station_id"] == 2].plot(x="stoptime10", y="dist", figsize=(14,4), kind="area");


# Let's build the features.

# In[16]:


features = df.pivot_table(index=["to_station_id", "to_station_name", "stopweekday"],
                          columns="stoptime10", values="dist").reset_index()
features.head()


# In[17]:


names = features.columns[3:]
names


# In[18]:


from sklearn.cluster import KMeans
clus = KMeans(8)
clus.fit(features[names])


# In[19]:


pred = clus.predict(features[names])
set(pred)


# In[20]:


features["cluster"] = pred


# Let's see what it means accross day.

# In[21]:


features[["cluster", "stopweekday", "to_station_id"]].groupby(["cluster", "stopweekday"]).count()


# Let's draw the clusters.

# In[22]:


centers = clus.cluster_centers_.T
import matplotlib.pyplot as plt
fig, ax = plt.subplots(centers.shape[1], 1, figsize=(10,10))
x = list(range(0,centers.shape[0]))
for i in range(centers.shape[1]):
    ax[i].bar (x, centers[:,i], width=1.0)
    ax[i].set_ylabel("cluster %d" % i)


# Three patterns emerge. However, small clusters are still annoying but let's show them on a map.

# ## Graph
# 
# We first need to get 7 clusters for each stations, one per day.

# In[23]:


piv = features.pivot_table(index=["to_station_id","to_station_name"], columns="stopweekday", values="cluster")
piv.head()


# In[24]:


piv["distincts"] = piv.apply(lambda row: len(set(row[i] for i in range(0,7))), axis=1)


# Let's see which station is classified in more than 4 clusters. NaN means no bikes stopped at this stations. They are mostly unused stations.

# In[25]:


piv[piv.distincts >= 4]


# Let's draw a map on a week day.

# In[26]:


data = stations.merge(piv.reset_index(), left_on=["id", "name"],
                      right_on=["to_station_id", "to_station_name"], suffixes=('', '_c'))
data.sort_values("id").head()


# In[27]:


from ensae_projects.datainc.data_bikes import folium_html_stations_map

colors = ["blue", "red", "yellow", "gray", "green", "black", "brown"]
xy = []
for els in data.apply(lambda row: (row["latitude"], row["longitude"], row[1], row["name"]), axis=1):
    try:
        cl = int(els[2])
    except:
        # NaN
        continue
    name = "%s c%d" % (els[3], cl)
    color = colors[cl % len(colors)]
    xy.append( ( (els[0], els[1]), (name, color)))
folium_html_stations_map(xy, width="80%")


# We notice than people got to the park after work. Let's see during the week-end.

# In[28]:


from ensae_projects.datainc.data_bikes import folium_html_stations_map

colors = ["blue", "red", "yellow", "gray", "green", "black", "brown"]
xy = []
for els in data.apply(lambda row: (row["latitude"], row["longitude"], row[5], row["name"]), axis=1):
    try:
        cl = int(els[2])
    except:
        # NaN
        continue
    name = "%s c%d" % (els[3], cl)
    color = colors[cl % len(colors)]
    xy.append( ( (els[0], els[1]), (name, color)))
folium_html_stations_map(xy, width="80%")


# In[29]:





# In[30]: